# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import GameState
import GridUtils

import itertools

DIAGONAL_DIRECTIONS = (None, None, ((1,1),(1,-1)), ((1,1,1),(1,1,-1),(1,-1,1),(1,-1,-1)))

def diagonal_score(grid, pos, pnum):
    # only 2d and 3d for now
    dirs = DIAGONAL_DIRECTIONS[grid.ndim]
    total = 0
    for d in dirs:
        good = False
        num_in_line = 1 # account for pos
        # check forward and back, can't have value of 0, must have at least 1 (pos won't be in the iteration)
        line_positions = (GridUtils.enumerate_values_with_direction(grid, pos, d),
                          GridUtils.enumerate_values_with_direction(grid, pos, GridUtils.negate_tuple(d)))
        for p,val in itertools.chain(*line_positions):
            good = True
            if val==0:
                good = False
                break
            elif val==pnum:
                num_in_line += 1
        if good:
            total += num_in_line
    return total


class DiagonalsGameState(GameState):
    def reset(self):
        GameState.reset(self)
        self.extra_state = {'player_scores': [0,0]}

    def player_score(self, pnum):
        return self.extra_state['player_scores'][pnum-1]
        
    def copy_extra_state(self):
        return {'player_scores': self.extra_state['player_scores'][:]}

    def successors(self):
        for c in self.enumerate_coords_with_value(0):
            nextgrid = self.grid.copy()
            nextgrid[c] = self.current_player
            nextstate = self.next_state_with_grid(nextgrid)
            # add score for this move
            nextstate.extra_state['player_scores'][self.current_player-1] += diagonal_score(nextgrid, c, self.current_player)
            yield ( (c,), nextstate )
            

